@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2019-2022, Intel Corporation
+* Copyright (c) 2019-2024, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -46,7 +46,8 @@
 #include "mos_context_specific_next.h"
 #include "mos_gpucontextmgr_next.h"
 #include "mos_cmdbufmgr_next.h"
-#include "media_user_settings_mgr.h"
+#include "mos_oca_rtlog_mgr.h"
+#include "mos_oca_interface_specific.h"
 #define BATCH_BUFFER_SIZE 0x80000
 
 OsContextSpecificNext::OsContextSpecificNext()
@@ -62,9 +63,10 @@ OsContextSpecificNext::~OsContextSpecificNext()
 MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
 {
     uint32_t      iDeviceId = 0;
-    MOS_STATUS    eStatus;
-    uint32_t      i = 0;
-    MOS_USER_FEATURE_VALUE_DATA UserFeatureData;
+    MOS_STATUS    eStatus   = MOS_STATUS_SUCCESS;
+    uint32_t      value     = 0;
+    uint32_t      mode      = 0;
+    MediaUserSettingSharedPtr   userSettingPtr = nullptr;
 
     MOS_OS_FUNCTION_ENTER;
 
@@ -74,45 +76,57 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
 
     if (GetOsContextValid() == false)
     {
+        uint32_t commandBufferSize = 0;
         m_skuTable.reset();
         m_waTable.reset();
         MosUtilities::MosZeroMemory(&m_platformInfo, sizeof(m_platformInfo));
         MosUtilities::MosZeroMemory(&m_gtSystemInfo, sizeof(m_gtSystemInfo));
 
         if( nullptr == osDriverContext  ||
-            0 >= osDriverContext->fd )
+            0 > osDriverContext->fd )
         {
             MOS_OS_ASSERT(false);
             return MOS_STATUS_INVALID_HANDLE;
         }
         m_fd = osDriverContext->fd;
 
-        m_bufmgr = mos_bufmgr_gem_init(m_fd, BATCH_BUFFER_SIZE);
+        userSettingPtr = MosInterface::MosGetUserSettingInstance(osDriverContext);
+
+        m_bufmgr = mos_bufmgr_gem_init(m_fd, BATCH_BUFFER_SIZE, &m_deviceType);
         if (nullptr == m_bufmgr)
         {
             MOS_OS_ASSERTMESSAGE("Not able to allocate buffer manager, fd=0x%d", m_fd);
             return MOS_STATUS_INVALID_PARAMETER;
         }
-        mos_bufmgr_gem_enable_reuse(m_bufmgr);
+        mos_bufmgr_enable_reuse(m_bufmgr);
 
         osDriverContext->bufmgr                 = m_bufmgr;
 
         //Latency reducation:replace HWGetDeviceID to get device using ioctl from drm.
-        iDeviceId   = mos_bufmgr_gem_get_devid(m_bufmgr);
+        iDeviceId   = mos_bufmgr_get_devid(m_bufmgr);
         m_isAtomSOC = IS_ATOMSOC(iDeviceId);
 
-        eStatus = NullHW::Init(osDriverContext);
-        if (!NullHW::IsEnabled())
+        eStatus = NullHwInit((MOS_CONTEXT_HANDLE)osDriverContext);
+        if (!GetNullHwIsEnabled())
         {
-            eStatus = HWInfo_GetGfxInfo(m_fd, m_bufmgr, &m_platformInfo, &m_skuTable, &m_waTable, &m_gtSystemInfo);
+            eStatus = HWInfo_GetGfxInfo(m_fd, m_bufmgr, &m_platformInfo, &m_skuTable, &m_waTable, &m_gtSystemInfo, userSettingPtr);
         }
         else
         {
-            m_platformInfo = osDriverContext->platform;
-            m_skuTable = osDriverContext->SkuTable;
-            m_waTable = osDriverContext->WaTable;
-            m_gtSystemInfo = osDriverContext->gtSystemInfo;
-            iDeviceId = osDriverContext->iDeviceId;
+            m_platformInfo  = osDriverContext->m_platform;
+            m_skuTable      = osDriverContext->m_skuTable;
+            m_waTable       = osDriverContext->m_waTable;
+            m_gtSystemInfo  = osDriverContext->m_gtSystemInfo;
+            iDeviceId       = osDriverContext->iDeviceId;
+        }
+
+        // replace platform/sku/wa/gtsysinfo for os context
+        if (Mos_Solo_IsEnabled(nullptr))
+        {
+            m_skuTable.reset();
+            m_waTable.reset();
+            Mos_Solo_SetPlatform(&m_platformInfo, (PMOS_CONTEXT)osDriverContext);
+            Mos_Solo_SetSkuwaGtInfo((PMOS_CONTEXT)osDriverContext, m_platformInfo, m_skuTable, m_waTable, m_gtSystemInfo, iDeviceId);
         }
 
         if (eStatus != MOS_STATUS_SUCCESS)
@@ -121,13 +135,35 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
             return eStatus;
         }
 
-        MOS_ZeroMemory(&UserFeatureData, sizeof(UserFeatureData));
-        MOS_UserFeature_ReadValue_ID(
-            nullptr,
-            __MEDIA_USER_FEATURE_VALUE_ENABLE_SOFTPIN_ID,
-            &UserFeatureData,
-            osDriverContext);
-        if (UserFeatureData.i32Data)
+        if (m_platformInfo.eProductFamily == IGFX_METEORLAKE ||
+            m_platformInfo.eProductFamily == IGFX_ARROWLAKE ||
+            m_platformInfo.eProductFamily == IGFX_LUNARLAKE)
+        {
+            ReadUserSetting(
+                userSettingPtr,
+                value,
+                "INTEL MEDIA ALLOC MODE",
+                MediaUserSetting::Group::Device);
+
+            if (value)
+            {
+                mode = (value & 0x000000ff);
+            }
+
+            // Realloc cache only if it's not mode 0
+            if (mode)
+            {
+                mos_bufmgr_realloc_cache(m_bufmgr, mode);
+            }
+        }
+
+        ReadUserSetting(
+            userSettingPtr,
+            value,
+            __MEDIA_USER_FEATURE_VALUE_ENABLE_SOFTPIN,
+            MediaUserSetting::Group::Device);
+
+        if (value)
         {
             bool softpin_va1Malign = false;
             if (MEDIA_IS_SKU(&m_skuTable, Ftr1MGranularAuxTable))
@@ -135,7 +171,36 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
                 softpin_va1Malign = true;
             }
 
-            mos_bufmgr_gem_enable_softpin(m_bufmgr, softpin_va1Malign);
+            mos_bufmgr_enable_softpin(m_bufmgr, softpin_va1Malign);
+
+            ReadUserSetting(
+                userSettingPtr,
+                value,
+                __MEDIA_USER_FEATURE_VALUE_ENABLE_VM_BIND,
+                MediaUserSetting::Group::Device);
+
+            if (value)
+            {
+                 mos_bufmgr_enable_vmbind(m_bufmgr);
+                 MOS_OS_NORMALMESSAGE("mos_bufmg_enable_vmbind");
+            }
+        }
+
+        uint64_t isRecoverableContextEnabled = 0;
+        MOS_LINUX_CONTEXT *intel_context = mos_context_create_ext(m_bufmgr, 0, false);
+        int ret = mos_get_context_param(intel_context, 0, DRM_CONTEXT_PARAM_RECOVERABLE, &isRecoverableContextEnabled);
+        if (ret == -EINVAL)
+        {
+            isRecoverableContextEnabled = 1;
+        }
+        if (intel_context)
+        {
+            mos_context_destroy(intel_context);
+        }
+        // set recoverablecontext disabled if want disable object capture
+        if (MEDIA_IS_WA(&m_waTable, WaDisableSetObjectCapture) && isRecoverableContextEnabled)
+        {
+            mos_bufmgr_disable_object_capture(m_bufmgr);
         }
 
         if (MEDIA_IS_SKU(&m_skuTable, FtrEnableMediaKernels) == 0)
@@ -143,8 +208,6 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
             MEDIA_WR_WA(&m_waTable, WaHucStreamoutOnlyDisable, 0);
         }
 
-        MediaUserSettingsMgr::MediaUserSettingsInit(m_platformInfo.eProductFamily);
-
         MosUtilities::MosTraceSetupInfo(
             (VA_MAJOR_VERSION << 16) | VA_MINOR_VERSION,
             m_platformInfo.eProductFamily,
@@ -155,7 +218,7 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
         GMM_WA_TABLE            gmmWaTable    = {};
         GMM_GT_SYSTEM_INFO      gmmGtInfo     = {};
         GMM_ADAPTER_BDF         gmmAdapterBDF = {};
-        eStatus = HWInfo_GetGmmInfo(m_fd, &gmmSkuTable, &gmmWaTable, &gmmGtInfo);
+        eStatus = HWInfo_GetGmmInfo(m_bufmgr, &gmmSkuTable, &gmmWaTable, &gmmGtInfo);
         if (MOS_STATUS_SUCCESS != eStatus)
         {
             MOS_OS_ASSERTMESSAGE("Fatal error - unsuccesfull Gmm Sku/Wa/GtSystemInfo initialization");
@@ -179,6 +242,14 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
         gmmInitAgrs.FileDescriptor    = gmmAdapterBDF.Data;
         gmmInitAgrs.ClientType        = (GMM_CLIENT)GMM_LIBVA_LINUX;
 
+        // replace sku/wa/gtsysinfo for gmm client context
+        if (Mos_Solo_IsEnabled(nullptr))
+        {
+            gmmInitAgrs.pSkuTable         = &m_skuTable;
+            gmmInitAgrs.pWaTable          = &m_waTable;
+            gmmInitAgrs.pGtSysInfo        = &m_gtSystemInfo;
+        }
+
         GMM_STATUS status = InitializeGmm(&gmmInitAgrs, &gmmOutArgs);
         if (status != GMM_SUCCESS)
         {
@@ -189,15 +260,13 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
 
         m_auxTableMgr = AuxTableMgr::CreateAuxTableMgr(m_bufmgr, &m_skuTable, m_gmmClientContext);
 
-        MOS_ZeroMemory(&UserFeatureData, sizeof(UserFeatureData));
 #if (_DEBUG || _RELEASE_INTERNAL)
-        MOS_UserFeature_ReadValue_ID(
-            nullptr,
-            __MEDIA_USER_FEATURE_VALUE_SIM_ENABLE_ID,
-            &UserFeatureData,
-            osDriverContext);
+        ReadUserSettingForDebug(
+            userSettingPtr,
+            osDriverContext->bSimIsActive,
+            __MEDIA_USER_FEATURE_VALUE_SIM_ENABLE,
+            MediaUserSetting::Group::Device);
 #endif
-        osDriverContext->bSimIsActive = (int32_t)UserFeatureData.i32Data;
 
         m_useSwSwizzling = osDriverContext->bSimIsActive || MEDIA_IS_SKU(&m_skuTable, FtrUseSwSwizzling);
 
@@ -205,13 +274,13 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
 
         m_use64BitRelocs = true;
 
-        if (!NullHW::IsEnabled())
+        if (!GetNullHwIsEnabled())
         {
             osDriverContext->iDeviceId              = iDeviceId;
-            osDriverContext->SkuTable               = m_skuTable;
-            osDriverContext->WaTable                = m_waTable;
-            osDriverContext->gtSystemInfo           = m_gtSystemInfo;
-            osDriverContext->platform               = m_platformInfo;
+            osDriverContext->m_skuTable             = m_skuTable;
+            osDriverContext->m_waTable              = m_waTable;
+            osDriverContext->m_gtSystemInfo         = m_gtSystemInfo;
+            osDriverContext->m_platform             = m_platformInfo;
         }
         osDriverContext->pGmmClientContext      = m_gmmClientContext;
         osDriverContext->m_auxTableMgr          = m_auxTableMgr;
@@ -225,9 +294,18 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
 
         SetOsContextValid(true);
         // Prepare the command buffer manager
+        if (m_ocaLogSectionSupported)
+        {
+            // increase size for oca log section
+            commandBufferSize = MosOcaInterfaceSpecific::IncreaseSize(COMMAND_BUFFER_SIZE);
+        }
+        else
+        {
+            commandBufferSize = COMMAND_BUFFER_SIZE;
+        }
         m_cmdBufMgr = CmdBufMgrNext::GetObject();
         MOS_OS_CHK_NULL_RETURN(m_cmdBufMgr);
-        MOS_OS_CHK_STATUS_RETURN(m_cmdBufMgr->Initialize(this, COMMAND_BUFFER_SIZE/2));
+        MOS_OS_CHK_STATUS_RETURN(m_cmdBufMgr->Initialize(this, commandBufferSize));
 
         // Prepare the gpu Context manager
         m_gpuContextMgr = GpuContextMgrNext::GetObject(this);
@@ -238,7 +316,7 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
         osDriverContext->pPerfData = m_perfData;
 
         //It must be done with m_gpuContextMgr ready. Insides it will create gpu context.
-#ifdef _MMC_SUPPORTED
+
         m_mosDecompression = MOS_New(MosDecompression, osDriverContext);
         MOS_OS_CHK_NULL_RETURN(m_mosDecompression);
         osDriverContext->ppMediaMemDecompState = m_mosDecompression->GetMediaMemDecompState();
@@ -247,7 +325,7 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
         {
             MOS_OS_NORMALMESSAGE("Decomp state creation failed");
         }
-#endif
+
         m_mosMediaCopy = MOS_New(MosMediaCopy, osDriverContext);
         if (nullptr == m_mosMediaCopy)
         {
@@ -262,6 +340,7 @@ MOS_STATUS OsContextSpecificNext::Init(DDI_DEVICE_CONTEXT ddiDriverContext)
             }
         }
     }
+    MosOcaRTLogMgr::RegisterContext(this, osDriverContext);
     return eStatus;
 }
 
